﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Interfaces;

namespace DAL.Repositories
{
    public class EFRepository<TEntity> : IRepository<TEntity>
        where TEntity : class
    {
        protected DbContext RepositoryDbContext;
        protected DbSet<TEntity> RepositoryDbSet;

        public EFRepository(DbContext dbContext)
        {
            if (dbContext == null)
            {
                throw new ArgumentNullException(paramName: nameof(dbContext));
            }
            RepositoryDbContext = dbContext;
            RepositoryDbSet = RepositoryDbContext.Set<TEntity>();
            if (RepositoryDbSet == null)
            {
                throw new NullReferenceException(message: nameof(RepositoryDbSet));
            }
        }

        public List<TEntity> All
        {
            get
            {
                var resCount = RepositoryDbSet.Count();
                if (resCount < 10)
                {
                    return RepositoryDbSet.ToList();
                }
                throw new Exception(message: "WTF? To many record in resultset! Please add custom data access method to repositry!");
            }
        }

        public TEntity Find(int id)
        {
            return RepositoryDbSet.Find(id);
        }

        public void Remove(int id)
        {
            Remove(entity: Find(id: id));
        }

        public void Remove(TEntity entity)
        {
            RepositoryDbSet.Remove(entity: entity);
        }

        public TEntity Add(TEntity entity)
        {
            return RepositoryDbSet.Add(entity: entity);
        }

        public void Update(TEntity entity)
        {
            RepositoryDbSet.Attach(entity: entity);
        }

        public int SaveChanges()
        {
            return RepositoryDbContext.SaveChanges();
        }
    }
}
